//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// 各種ビット操作
//

#pragma once

#include "header.h"

#if defined(HAVE___BUILTIN_BITREVERSE8)
#define bitrev(x)	__builtin_bitreverse8(x)
#else
extern const uint8 bitrev_table[256];

// 8ビットを左右反転する
static inline uint8
bitrev(uint8 x)
{
	return bitrev_table[x];
}
#endif

// 左ローテート
#if defined(HAVE___BUILTIN_ROTATELEFT32)
#define ROL32(a, n)	__builtin_rotateleft32(a, n)
#else
static inline uint32
ROL32(uint32 a, int n)
{
	n &= 31;
	if (__predict_false(n == 0)) {
		return a;
	} else {
		return (a << n) | (a >> (32 - n));
	}
}
#endif

// 右ローテート
#if defined(HAVE___BUILTIN_ROTATERIGHT32)
#define ROR32(a, n)	__builtin_rotateright32(a, n)
#else
static inline uint32
ROR32(uint32 a, int n)
{
	n &= 31;
	if (__predict_false(n == 0)) {
		return a;
	} else {
		return (a >> n) | (a << (32 - n));
	}
}
#endif

// mask で指定したいずれかのビットが 0 -> 1 に変化したら true
static constexpr bool
bit_rising(uint before, uint after, uint mask)
{
	// before が 0 で mask が 1 のビットだけを立てる。
	// それが 1 で after が 1 なら立ち上がり。
	return (~before & mask) & after;
}

// mask で指定したいずれかのビットが 1 -> 0 に変化したら true
static constexpr bool
bit_falling(uint before, uint after, uint mask)
{
	// before が 1 で mask が 1 のビットだけを立てる。
	// それが 1 で after が 0 なら立ち下がり。
	return (before & mask) & ~after;
}
