//
// nono
// Copyright (C) 2024 nono project
// Licensed under nono-license.txt
//

//
// VirtIO エントロピーデバイス
//

#include "virtio_entropy.h"
#include "virtio_def.h"
#include "monitor.h"

// コンストラクタ
VirtIOEntropyDevice::VirtIOEntropyDevice(uint slot_)
	: inherited(OBJ_VIRTIO_ENTROPY, slot_)
{
	// 短縮形
	AddAlias("vent");

	device_id = VirtIO::DEVICE_ID_ENTROPY;
	int num_max = 4;
	vqueues.emplace_back(this, 0, "RequestQ", num_max);

	// 割り込み名
	strlcpy(intrname, "VIOEnt", sizeof(intrname));
	// 完了通知メッセージ
	msgid = MessageID::VIRTIO_ENTROPY_DONE;

	monitor = gMonitorManager->Regist(ID_MONITOR_VIRTIO_ENTROPY, this);
	monitor->func = ToMonitorCallback(&VirtIOEntropyDevice::MonitorUpdate);
	monitor->SetSize(MONITOR_WIDTH, 2 + 4 * 1 + num_max + 3);
}

// デストラクタ
VirtIOEntropyDevice::~VirtIOEntropyDevice()
{
}

void
VirtIOEntropyDevice::MonitorUpdate(Monitor *, TextScreen& screen)
{
	int y = 0;

	screen.Clear();

	y = MonitorUpdateDev(screen, y);
	y++;
	y = MonitorUpdateVirtQueue(screen, y, vqueues[0]);
	y++;
	y = MonitorUpdateVirtQDesc(screen, y, vqueues[0]);
}

// ディスクリプタを一つ処理する。
void
VirtIOEntropyDevice::ProcessDesc(VirtIOReq& req)
{
	uint32 data = 0;

	while (req.pos < req.len) {
		if ((req.pos & 3) == 0) {
			data = rnd();
		}
		ReqWriteU8(req, data & 0xff);
		data >>= 8;
	}
}
