//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// VM (X68030)
//

#include "vm_x68k.h"
#include "config.h"
#include "human68k.h"
#include "mainapp.h"
#include "mainbus_x68k.h"
#include "mpu680x0.h"
#include "nmi.h"
#include "power.h"
#include "renderer.h"
#include "x68kkbd.h"

// コンストラクタ
VM_X68030::VM_X68030()
	: inherited("X68030")
{
	// 機種固有パラメータ
	// MPU 種別。
	gConfig->SetDefault("mpu-type", "68030");
	// MPU クロックの初期値は機種ごとに異なる。単位は MHz
	gConfig->SetDefault("mpu-clock", 25);
	// RAM 初期値は MAX にしておく
	gConfig->SetDefault("ram-size", 12);
	// SPC の入力クロック (5MHz = 200ns)
	gConfig->Add("!spc-tCLF", 200);

	// デバイス
	NEWDV(Power, new X68030PowerDevice());
	NEWDV(MPU, NewMPU680x0Device());
	NEWDV(Mainbus, new X68kMainbus());
	NEWDV(Keyboard, new X68030Keyboard());
	NEWDV(NMI, new NMIDevice());
	NEWDV(Renderer, new X68030Renderer());

	if (gMainApp.human_mode) {
		NEWDV(Human68k, new Human68kDevice());
	}

	// バーストアクセスなし (回路図より)
	auto mpu680x0 = GetMPU680x0Device(pMPU.get());
	if (mpu680x0) {
		mpu680x0->EnableBurstAccess(false);
	}
}

// デストラクタ
VM_X68030::~VM_X68030()
{
}
